from django.template.loaders import filesystem as fs
from django.template import TemplateDoesNotExist
from django.template.utils import get_app_template_dirs


class FilesystemLoader(fs.Loader):

    def get_contents(self, origin):
        # ODT is a compressed format not a string
        # we need to read the template w/o interpreting as byte stream
        try:
            with open(origin.name, 'rb') as fp:
                return fp.read()
        except FileNotFoundError:
            raise TemplateDoesNotExist(origin)

    def get_template_sources(self, template_name):
        """
        Return an Origin object pointing to an absolute path in each directory
        in template_dirs. For security reasons, if a path doesn't lie inside
        one of the template_dirs it is excluded from the result set.
        """
        # no point in searching if this is the wrong loader
        # this way we can set AppyPodEngine in first position in TEMPLATES
        if not (template_name.endswith('.odt') or template_name.endswith('.ods')):
            raise TemplateDoesNotExist(template_name)

        for template_dir in self.get_dirs():
            try:
                name = fs.safe_join(template_dir, template_name)
            except fs.SuspiciousFileOperation:
                # The joined path was located outside of this template_dir
                # (it might be inside another one, so this isn't fatal).
                continue

            yield fs.Origin(
                name=name,
                template_name=template_name,
                loader=self,
            )


class AppDirectoriesLoader(FilesystemLoader):

    def get_dirs(self):
        return get_app_template_dirs('templates')
